# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os.path

from oslo_config import cfg
from oslo_log import log

from kolla_kubernetes.common import file_utils
from kolla_kubernetes import exception

CONF = cfg.CONF
CNF_FIELDS = ('source', 'dest', 'owner', 'perm')
CMD_FIELDS = ('run_once', 'dependencies', 'command', 'env',
              'delay', 'retries', 'files')
DEP_FIELDS = ('path', 'scope')
SCOPE_OPTS = ('global', 'local')
LOG = log.getLogger()

# TODO(rhallisey): make pod definitions dynamic
# The pods associated with a service
POD_DEFINITIONS = {'mariadb': ['mariadb'],
                   'memcached': ['memcached'],
                   'rabbitmq': ['rabbitmq'],
                   'keystone': ['keystone'],
                   'glance': ['glance'],
                   'nova': ['nova-compute', 'nova-control'],
                   'neutron': ['neutron-compute', 'neutron-control',
                               'neutron-network'],
                   'swift': ['swift-account', 'swift-container',
                             'swift-object', 'swift-proxy']}

# TODO(rhallisey): make container definitions dynamic
# The containers in a pod
CONTAINER_DEFINITIONS = {'mariadb': ['mariadb'],
                         'memcached': ['memcached'],
                         'rabbitmq': ['rabbitmq'],
                         'keystone': ['keystone'],
                         'glance': ['glance-api', 'glance-registry'],
                         'nova-compute': ['nova-compute', 'nova-libvirt'],
                         'nova-control': ['nova-api', 'nova-scheduler',
                                          'nova-conductor'],
                         'neutron-compute': ['openvswitch-db-server',
                                             'openvswitch-vswitchd',
                                             'neutron-openvswitch-agent',
                                             'neutron-linuxbridge-agent'],
                         'neutron-network': ['neutron-l3-agent',
                                             'neutron-dhcp-agent',
                                             'neutron-medadata-agent'],
                         'neutron-control': ['neutron-server'],
                         'swift-account': ['swift-rsyncd',
                                           'swift-account-server',
                                           'swift-account-auditor',
                                           'swift-account-replicator',
                                           'swift-account-reaper'],
                         'swift-container': ['swift-rsyncd',
                                             'swift-container-server',
                                             'swift-container-auditor',
                                             'swift-container-replicator',
                                             'swift-container-updater'],
                         'swift-object': ['swift-rsyncd',
                                          'swift-object-server',
                                          'swift-object-auditor',
                                          'swift-object-replicator',
                                          'swift-object-updater',
                                          'swift-object-expirer'],
                         'swift-proxy': ['swift-proxy-server']}


def get_pod_definition(service):
    return POD_DEFINITIONS[service]


def get_container_definition(container):
    return CONTAINER_DEFINITIONS[container]


def get_services_directory():
    return (CONF.service_dir or
            os.path.join(file_utils.find_base_dir(), 'services'))


def find_service_files(service_name):
    service_dir = get_services_directory()
    LOG.debug('Looking for services files in %s', service_dir)

    if not os.path.exists(service_dir):
        raise exception.KollaNotFoundException(service_dir,
                                               entity='service directory')

    bootstrap_dir = os.path.join(service_dir, '../bootstrap/')
    LOG.debug('Looking for bootstrap files in %s', service_dir)
    if not os.path.exists(bootstrap_dir):
        raise exception.KollaNotFoundException(bootstrap_dir,
                                               entity='bootstrap directory')

    short_name = service_name.split('/')[-1]
    files = []
    for root, dirs, names in os.walk(service_dir):
        for name in names:
            if short_name in name:
                files.append(os.path.join(root, name))

    for root, dirs, names in os.walk(bootstrap_dir):
        for name in names:
            if short_name in name:
                files.append(os.path.join(root, name))

    if not files:
        raise exception.KollaNotFoundException(service_dir,
                                               entity='service definition')
    return files



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

from kolla_kubernetes.common import utils

CONF = cfg.CONF

kolla_opts = [
    cfg.StrOpt('namespace',
               default='kollaglue',
               help='The Docker namespace name'),
    cfg.StrOpt('tag',
               default='1.0.0',
               help='The Docker tag'),
    cfg.StrOpt('base',
               default='centos',
               help='The base distro which was used to build images'),
    cfg.StrOpt('base-tag',
               default='latest',
               help='The base distro image tag'),
    cfg.StrOpt('install-type',
               default='binary',
               help='The method of the OpenStack install'),
    cfg.StrOpt('deployment-id',
               default=utils.env('USER', default='default'),
               help='Uniq name for deployment'),
    cfg.StrOpt('profile',
               default='default',
               help='Build profile which was used to build images')
]
kolla_opt_group = cfg.OptGroup(name='kolla',
                               title='Options for Kolla Docker images')
CONF.register_group(kolla_opt_group)
CONF.register_cli_opts(kolla_opts, kolla_opt_group)
CONF.register_opts(kolla_opts, kolla_opt_group)

kubernetes_opts = [
    cfg.StrOpt('host', default='http://localhost:8080'),
    cfg.StrOpt('kubectl_path', default='kubectl')
]
kubernetes_opt_group = cfg.OptGroup(name='kolla_kubernetes')
CONF.register_group(kubernetes_opt_group)
CONF.register_cli_opts(kubernetes_opts, kubernetes_opt_group)
CONF.register_opts(kubernetes_opts, kubernetes_opt_group)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections

from kolla_kubernetes.common import jinja_utils
from kolla_kubernetes.tests import base


class TestJinjaUtils(base.BaseTestCase):

    def test_dict_jinja_render(self):
        raw_dict = collections.OrderedDict([
            ('first_key', '{{ test_var }}_test',),
            ('second_key', '{{ first_key }}_test'),
        ])
        jvars = {'test_var': 'test'}
        jinja_utils.dict_jinja_render(raw_dict, jvars)
        self.assertEqual(jvars['first_key'], 'test_test')
        self.assertEqual(jvars['second_key'], 'test_test_test')



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os


def env(*args, **kwargs):
    for arg in args:
        value = os.environ.get(arg)
        if value:
            return value
    return kwargs.get('default', '')


class JvarsDict(dict):
    """Dict which can contain the 'global_vars' which are always preserved.

    They cannot be be overriden by any update nor single item setting.
    """

    def __init__(self, *args, **kwargs):
        super(JvarsDict, self).__init__(*args, **kwargs)
        self.global_vars = {}

    def __setitem__(self, key, value, force=False):
        if not force and key in self.global_vars:
            return
        return super(JvarsDict, self).__setitem__(key, value)

    def set_force(self, key, value):
        """Sets the variable even if it will override a global variable."""
        return self.__setitem__(key, value, force=True)

    def update(self, other_dict, force=False):
        if not force:
            other_dict = {key: value for key, value in other_dict.items()
                          if key not in self.global_vars}
        super(JvarsDict, self).update(other_dict)

    def set_global_vars(self, global_vars):
        self.update(global_vars)
        self.global_vars = global_vars



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from kolla_kubernetes.common import file_utils
from kolla_kubernetes.tests import base


class FindBaseDirTest(base.BaseTestCase):

    def test_when_is_a_test(self):
        tdir = file_utils.find_base_dir()
        self.assertIsNotNone(tdir)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from kolla_kubernetes.common import type_utils
from kolla_kubernetes.tests import base


class StrToBoolTest(base.BaseTestCase):

    scenarios = [
        ('none', dict(text=None, expect=False)),
        ('empty', dict(text='', expect=False)),
        ('junk', dict(text='unlikely', expect=False)),
        ('no', dict(text='no', expect=False)),
        ('yes', dict(text='yes', expect=True)),
        ('0', dict(text='0', expect=False)),
        ('1', dict(text='1', expect=False)),
        ('True', dict(text='True', expect=True)),
        ('False', dict(text='False', expect=False)),
        ('true', dict(text='true', expect=True)),
        ('false', dict(text='false', expect=False)),
        ('shouty', dict(text='TRUE', expect=True)),
    ]

    def test_str_to_bool(self):
        self.assertEqual(self.expect, type_utils.str_to_bool(self.text))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class KollaException(Exception):
    pass


class KollaDirNotFoundException(KollaException):
    pass


class KollaNotFoundException(KollaException):
    def __init__(self, message, entity='file'):
        super(KollaNotFoundException, self).__init__(
            'The %s "%s" was not found' % (entity, message))


class KollaNotSupportedException(KollaNotFoundException):
    def __init__(self, operation='update', entity='kubernetes'):
        super(KollaNotFoundException, self).__init__(
            'Operation "%s" is not supported by "%s"' % (operation, entity))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import sys

from oslo_utils import importutils

from kolla_kubernetes import exception


LOG = logging.getLogger(__name__)


def find_config_file(filename):
    filepath = os.path.join('/etc/kolla', filename)
    if os.access(filepath, os.R_OK):
        return filepath

    filepath = os.path.join('/etc/kolla-kubernetes', filename)
    if os.access(filepath, os.R_OK):
        return filepath

    raise exception.KollaDirNotFoundException(
        'Unable to detect kolla-kubernetes directory'
    )


def get_service_config_files(service_name):
    directory = os.path.join('/etc/kolla/', service_name)
    for dirpath, _, filenames in os.walk(directory):
        for f in filenames:
            yield os.path.abspath(os.path.join(dirpath, f))


def get_src_dir():
    kolla_kubernetes = importutils.import_module('kolla_kubernetes')
    mod_path = os.path.abspath(kolla_kubernetes.__file__)
    # remove the file and module to get to the base.
    return os.path.dirname(os.path.dirname(mod_path))


def get_shared_directory():
    script_path = os.path.abspath(os.path.join(os.path.dirname(
        os.path.realpath(sys.argv[0])), '../..'))
    base_script_path = os.path.basename(script_path)

    if base_script_path == 'kolla-kubernetes':
        return script_path
    elif os.path.exists('/usr/local/share/kolla'):
        return '/usr/local/share/kolla'
    elif os.path.exists('/usr/share/kolla'):
        return '/usr/share/kolla'
    raise exception.KollaDirNotFoundException(
        'Unable to detect kolla-kubernetes directory'
    )


def find_base_dir():
    if os.path.exists('/usr/local/share/kolla'):
        return '/usr/local/share/kolla'
    elif os.path.exists('/usr/share/kolla'):
        return '/usr/share/kolla'
    else:
        return get_src_dir()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import os.path
import subprocess
import tempfile
import time

from oslo_config import cfg
from oslo_log import log as logging
import yaml

from kolla_kubernetes.common import file_utils
from kolla_kubernetes.common import jinja_utils
from kolla_kubernetes.common import utils
from kolla_kubernetes import service_definition

LOG = logging.getLogger()
CONF = cfg.CONF
CONF.import_group('kolla', 'kolla_kubernetes.config')
CONF.import_group('kolla_kubernetes', 'kolla_kubernetes.config')

PROJECT_ROOT = os.path.abspath(os.path.join(
    os.path.dirname(os.path.realpath(__file__)), '../..'))


def _create_working_directory():
    ts = time.time()
    ts = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d_%H-%M-%S_')
    temp_dir = tempfile.mkdtemp(prefix='kolla-' + ts)
    working_dir = os.path.join(temp_dir, 'kubernetes')
    os.makedirs(working_dir)
    return working_dir


def _load_variables_from_file(project_name):
    jvars = utils.JvarsDict()
    f = file_utils.find_config_file('kolla-kubernetes.yml')
    if os.path.exists(f):
        with open(f, 'r') as gf:
            jvars.set_global_vars(yaml.load(gf))
    else:
        LOG.warning('Unable to load %s', f)
    f = file_utils.find_config_file('globals.yml')
    if os.path.exists(f):
        with open(f, 'r') as gf:
            jvars.set_global_vars(yaml.load(gf))
    else:
        LOG.warning('Unable to load %s', f)
    f = file_utils.find_config_file('passwords.yml')
    if os.path.exists(f):
        with open(f, 'r') as gf:
            jvars.update(yaml.load(gf))
    else:
        LOG.warning('Unable to load %s', f)
    # Apply the basic variables that aren't defined in any config file.
    jvars.update({
        'deployment_id': CONF.kolla.deployment_id,
        'node_config_directory': '',
        'timestamp': str(time.time())
    })

    dir = file_utils.get_shared_directory()
    all_yml = os.path.join(dir, 'ansible/group_vars/all.yml')
    local_dir = os.path.join(PROJECT_ROOT, 'kolla/ansible/')

    if dir and os.path.exists(all_yml):
        jinja_utils.yaml_jinja_render(all_yml, jvars)
    elif dir and os.path.exists(local_dir):
        local_group_vars = os.path.join(local_dir, 'group_vars/all.yml')
        jinja_utils.yaml_jinja_render(local_group_vars, jvars)
    else:
        LOG.warning('Unable to load %s', all_yml)

    proj_ansible_roles = os.path.join(dir, 'ansible/roles', project_name,
                                      'defaults', 'main.yml')
    local_ansible_roles = os.path.join(local_dir, 'roles', project_name,
                                       'defaults', 'main.yml')

    if dir and os.path.exists(proj_ansible_roles):
        jinja_utils.yaml_jinja_render(proj_ansible_roles, jvars)
    elif dir and os.path.exists(local_ansible_roles):
        jinja_utils.yaml_jinja_render(local_ansible_roles, jvars)
    else:
        LOG.warning('Unable to load %s', proj_ansible_roles)

    common_ansible_roles = os.path.join(dir, 'ansible/roles', 'common',
                                        'defaults', 'main.yml')
    common_local_ansible_roles = os.path.join(local_dir, 'roles', 'common',
                                              'defaults', 'main.yml')

    if dir and os.path.exists(common_ansible_roles):
        jinja_utils.yaml_jinja_render(common_ansible_roles, jvars)
    elif dir and os.path.exists(common_local_ansible_roles):
        jinja_utils.yaml_jinja_render(common_local_ansible_roles, jvars)
    else:
        LOG.warning('Unable to load %s', common_ansible_roles)
    return jvars


def _build_runner(working_dir, service_name, pod_list, variables=None):
    for filename in service_definition.find_service_files(service_name):
        proj_filename = filename.split('/')[-1].replace('.j2', '')
        proj_name = filename.split('/')[-2]
        LOG.debug(
            'proj_filename : %s proj_name: %s' % (proj_filename, proj_name))

        variables = _load_variables_from_file(proj_name)

        content = yaml.load(
            jinja_utils.jinja_render(filename, variables))
        with open(os.path.join(working_dir, proj_filename), 'w') as f:
            LOG.debug('_build_runner : service file : %s' %
                      os.path.join(working_dir, proj_filename))
            f.write(yaml.dump(content, default_flow_style=False))


def run_service(service_name, variables=None):
    working_dir = _create_working_directory()
    pod_list = service_definition.get_pod_definition(service_name)
    _build_runner(working_dir, service_name, pod_list, variables=variables)
    _deploy_instance(working_dir, service_name, pod_list)


def kill_service(service_name, variables=None):
    working_dir = _create_working_directory()
    pod_list = service_definition.get_pod_definition(service_name)
    _build_runner(working_dir, service_name, pod_list, variables=variables)
    _delete_instance(working_dir, service_name, pod_list)


def _deploy_instance(directory, service_name, pod_list):
    server = "--server=" + CONF.kolla_kubernetes.host

    for pod in pod_list:
        container_list = service_definition.get_container_definition(pod)
        for container in container_list:
            cmd = [CONF.kolla_kubernetes.kubectl_path, server, "create",
                   "configmap", '%s-configmap' % container]
            cmd = cmd + ['--from-file=%s' % f
                         for f in
                         file_utils.get_service_config_files(container)]

            LOG.info('Command : %r' % cmd)
            subprocess.call(cmd)

    cmd = [CONF.kolla_kubernetes.kubectl_path, server, "create", "-f",
           directory]
    LOG.info('Command : %r' % cmd)
    subprocess.call(cmd)


def _delete_instance(directory, service_name, pod_list):
    server = "--server=" + CONF.kolla_kubernetes.host

    for pod in pod_list:
        container_list = service_definition.get_container_definition(pod)
        for container in container_list:
            cmd = [CONF.kolla_kubernetes.kubectl_path, server, "delete",
                   "configmap", '%s-configmap' % container]

            LOG.info('Command : %r' % cmd)
            subprocess.call(cmd)

    cmd = [CONF.kolla_kubernetes.kubectl_path, server, "delete", "-f",
           directory]
    LOG.info('Command : %r' % cmd)
    subprocess.call(cmd)



#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import shlex
import sys

from cliff import app
from cliff import commandmanager
from cliff import interactive
from oslo_config import cfg
from oslo_log import log

PROJECT_ROOT = os.path.abspath(os.path.join(
    os.path.dirname(os.path.realpath(__file__)), '../..'))

if PROJECT_ROOT not in sys.path:
    sys.path.insert(0, PROJECT_ROOT)

from kolla_kubernetes.common import utils

PROJECT = 'kolla_kubernetes'
VERSION = '1.0'

CONF = cfg.CONF
CONF.import_group('kolla', 'kolla_kubernetes.config')
CONF.import_group('kolla_kubernetes', 'kolla_kubernetes.config')

log.register_options(CONF)
log.set_defaults(
    default_log_levels='requests.packages.urllib3.connectionpool=WARNING')

cli_opts = [
    cfg.StrOpt('service-dir',
               default=utils.env('K8S_SERVICE_DIR'),
               help='Directory with services, (Env: K8S_SERVICE_DIR)'),
]
CONF.register_cli_opts(cli_opts)


class KollaKubernetesInteractiveApp(interactive.InteractiveApp):
    def do_run(self, arg):
        self.default(arg)

    def do_help(self, arg):
        line_parts = shlex.split(arg)
        try:
            self.command_manager.find_command(line_parts)
            return self.default(self.parsed('help ' + arg))
        except ValueError:
            # There is a builtin cmd2 command
            pass
        return interactive.InteractiveApp.do_help(self, arg)


class KollaKubernetesShell(app.App):
    def __init__(self):
        super(KollaKubernetesShell, self).__init__(
            description='Kolla-kubernetes command-line interface',
            version=VERSION,
            command_manager=commandmanager.CommandManager(
                'kolla_kubernetes.cli'),
            deferred_help=True,
            interactive_app_factory=KollaKubernetesInteractiveApp
        )

    def configure_logging(self):
        return

    def print_help(self):
        outputs = []
        max_len = 0
        self.stdout.write('\nCommands :\n')

        for name, ep in sorted(self.command_manager):
            factory = ep.load()
            cmd = factory(self, None)
            one_liner = cmd.get_description().split('\n')[0]
            outputs.append((name, one_liner))
            max_len = max(len(name), max_len)

        for name, one_liner in outputs:
            self.stdout.write('  %s  %s\n' % (name.ljust(max_len), one_liner))


def _separate_args(argv):
    conf_opts = _config_opts_map()
    config_args = []
    command_args = argv[:]
    while command_args:
        nargs = conf_opts.get(command_args[0])
        if nargs:
            config_args.extend(command_args[:nargs])
            command_args = command_args[nargs:]
        else:
            break
    return config_args, command_args


def _config_opts_map():
    opts = {'--help': 1, '-h': 1, '--config-dir': 2, '--config-file': 2,
            '--version': 1}
    for opt in CONF._all_cli_opts():
        if opt[1]:
            arg = '%s-%s' % (opt[1].name, opt[0].name)
        else:
            arg = opt[0].name

        if isinstance(opt[0], cfg.BoolOpt):
            nargs = 1
            opts['--no%s' % arg] = 1
        else:
            nargs = 2
        opts['--%s' % arg] = nargs

        if opt[0].short:
            opts['-%s' % opt[0].short] = nargs

        for dep_opt in opt[0].deprecated_opts:
            if getattr(dep_opt, 'group'):
                opts['--%s-%s' % (dep_opt.group, dep_opt.name)] = nargs
            else:
                opts['--%s' % dep_opt.name] = nargs

    return opts


def main(argv=sys.argv[1:]):
    config_args, command_args = _separate_args(argv)

    need_help = (['help'] == command_args or '-h' in config_args or
                 '--help' in config_args)
    if need_help:
        CONF([], project=PROJECT, version=VERSION)
        CONF.print_help()
        return KollaKubernetesShell().print_help()

    CONF(config_args, project=PROJECT, version=VERSION)
    log.setup(CONF, PROJECT, VERSION)

    if '-d' in config_args or '--debug' in config_args:
        command_args.insert(0, '--debug')
        CONF.log_opt_values(
            log.getLogger(PROJECT), log.INFO)

    return KollaKubernetesShell().run(command_args)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cliff import command
from oslo_config import cfg
from oslo_log import log


from kolla_kubernetes import service

CONF = cfg.CONF
LOG = log.getLogger(__name__)


class Run(command.Command):
    """Run a service."""

    def get_parser(self, prog_name):
        parser = super(Run, self).get_parser(prog_name)
        parser.add_argument('service')
        return parser

    def take_action(self, parsed_args):
        service.run_service(parsed_args.service)


class Kill(command.Command):
    """Kill a service."""

    def get_parser(self, prog_name):
        parser = super(Kill, self).get_parser(prog_name)
        parser.add_argument('service')
        return parser

    def take_action(self, parsed_args):
        service.kill_service(parsed_args.service)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import logging
import os

import jinja2
from jinja2 import meta
import six
import yaml

from kolla_kubernetes.common import type_utils

LOG = logging.getLogger(__name__)


# Customize PyYAML library to return the OrderedDict. That is needed, because
# when iterating on dict, we reuse its previous values when processing the
# next values and the order has to be preserved.

def ordered_dict_constructor(loader, node):
    """OrderedDict constructor for PyYAML."""
    return collections.OrderedDict(loader.construct_pairs(node))


def ordered_dict_representer(dumper, data):
    """Representer for PyYAML which is able to work with OrderedDict."""
    return dumper.represent_dict(data.items())


yaml.add_constructor(yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
                     ordered_dict_constructor)
yaml.add_representer(collections.OrderedDict, ordered_dict_representer)


def jinja_render(fullpath, global_config, extra=None):
    variables = global_config
    if extra:
        variables.update(extra)

    myenv = jinja2.Environment(
        loader=jinja2.FileSystemLoader(
            os.path.dirname(fullpath)))
    myenv.filters['bool'] = type_utils.str_to_bool
    return myenv.get_template(os.path.basename(fullpath)).render(variables)


def jinja_render_str(content, global_config, name='dafault_name', extra=None):
    variables = global_config
    if extra:
        variables.update(extra)

    myenv = jinja2.Environment(loader=jinja2.DictLoader({name: content}))
    myenv.filters['bool'] = type_utils.str_to_bool
    return myenv.get_template(name).render(variables)


def jinja_find_required_variables(fullpath):
    myenv = jinja2.Environment(loader=jinja2.FileSystemLoader(
        os.path.dirname(fullpath)))
    myenv.filters['bool'] = type_utils.str_to_bool
    template_source = myenv.loader.get_source(myenv,
                                              os.path.basename(fullpath))[0]
    parsed_content = myenv.parse(template_source)
    return meta.find_undeclared_variables(parsed_content)


def dict_jinja_render(raw_dict, jvars):
    """Renders dict with jinja2 using provided variables and itself.

    By using itself, we mean reusing the previous values from dict for the
    potential render of the next value in dict.
    """
    for key, value in raw_dict.items():
        if isinstance(value, six.string_types):
            value = jinja_render_str(value, jvars)
        elif isinstance(value, dict):
            value = dict_jinja_render(value, jvars)
        jvars[key] = value


def yaml_jinja_render(filename, jvars):
    """Parses YAML file and templates it with jinja2.

    1. YAML file is rendered by jinja2 based on the provided variables.
    2. Rendered file is parsed.
    3. The every element dictionary being a result of parsing is rendered again
       with itself.
    """
    with open(filename, 'r') as yaml_file:
        raw_dict = yaml.load(yaml_file)
    dict_jinja_render(raw_dict, jvars)
